/**
 * Namespace Api
 *
 * All backend api type
 */
declare namespace Api {
  namespace Common {
    /** common params of paginating */
    interface PaginatingCommonParams {
      /** current page number */
      current: number;
      /** page size */
      size: number;
      /** total count */
      total: number;
      count: number;
      pageIndex: number;
      pageSize: number;
      list: [];
    }

    /** common params of paginating query list data */
    interface PaginatingQueryRecord<T = any> extends PaginatingCommonParams {
      records: T[];
    }

    /**
     * enable status
     *
     * - "1": enabled
     * - "2": disabled
     */
    type EnableStatus = '1' | '2';

    /** common record */
    type CommonRecord<T = any> = {
      /** record id */
      id: number;
      /** record creator */
      createBy: string;
      /** record create time */
      createTime: string;
      /** record updater */
      updateBy: string;
      /** record update time */
      updateTime: string;
      /** record status */
      status: EnableStatus;
    } & T;
  }

  /**
   * Namespace Auth
   *
   * Backend api module: "auth"
   */
  namespace Auth {
    interface LoginToken {
      token: string;
      refreshToken: string;
      data: any;
      code: number;
      userName: string;
    }

    interface UserInfo {
      userId: string;
      userName: string;
      roles: string[];
      buttons: string[];
    }

    // 验证码
    interface CodeImg {
      code: Number;
      data: string;
      id: string;
      msg: string;
    }

    // 公钥
    interface Pubkey {
      code: Number;
      data: string;
      msg: string;
    }
  }

  /**
   * Namespace Route
   *
   * Backend api module: "route"
   */
  namespace Route {
    type ElegantConstRoute = import('@elegant-router/types').ElegantConstRoute;

    interface MenuRoute extends ElegantConstRoute {
      id: string;
    }

    interface UserRoute {
      routes: MenuRoute[];
      home: import('@elegant-router/types').LastLevelRouteKey;
    }
  }

  /**
   * namespace SystemManage
   *
   * backend api module: "systemManage"
   */
  namespace SystemManage {
    type CommonSearchParams = Pick<Common.PaginatingCommonParams, 'pageIndex' | 'pageSize'>;

    /** role */
    type Role = Common.CommonRecord<{
      /** role name */
      roleName: string;
      /** role code */
      roleKey: string;
      /** role description */
      status: string;
      roleSort: number | string;
      remark: string;
      menuIds: number[];
      roleId: number;
      dataScope: string;
    }>;

    /** role search params */
    type RoleSearchParams = Partial<
      Pick<Api.SystemManage.Role, 'roleName' | 'roleKey' | 'status' | 'roleSort' | 'remark' | 'menuIds'> &
        CommonSearchParams
    >;
    type ChangeRoleStatusParams = {
      roleId: number | string;
      status: string;
    };
    /** role list */
    type RoleList = Common.PaginatingQueryRecord<Role>;

    /** all role */
    type AllRole = Pick<Role, 'id' | 'roleName' | 'roleId' | 'roleKey' | 'status' | 'admin' | 'roleSort'>;

    /**
     * user gender
     *
     * - "1": "male"
     * - "2": "female"
     */
    type UserGender = '1' | '2';

    /** user */
    type User = Common.CommonRecord<{
      /** user name */
      username: string;
      userId: number;
      // 'deptName': string,
      deptId: number;
      /** user nick name */
      nickName: string;
      /** user phone */
      phone: string;
      /** user email */
      /** user role code collection */
      roleId: number;
      postId: number;
      remark: string;
      email: string;
      sex: string;
      status: string;
      password: string;
    }>;

    /** user search params */
    type UserSearchParams = Partial<
      Pick<
        Api.SystemManage.User,
        | 'username'
        | 'userId'
        | 'nickName'
        | 'phone'
        | 'status'
        | 'deptId'
        | 'postId'
        | 'remark'
        | 'email'
        | 'sex'
        | 'password'
      > &
        CommonSearchParams
    >;

    /** user list */
    type UserList = Common.PaginatingQueryRecord<User>;

    /**
     * menu type
     *
     * - "M": 目录
     * - "C": 菜单 F:按钮
     */
    type MenuType = 'C' | 'M' | 'F';

    type MenuButton = {
      /**
       * button code
       *
       * it can be used to control the button permission
       */
      code: string;
      /** button description */
      desc: string;
    };

    /**
     * icon type
     *
     * - "1": iconify icon
     * - "2": local icon
     */
    type IconType = '1' | '2';

    type MenuPropsOfRoute = Pick<
      import('vue-router').RouteMeta,
      'i18nKey' | 'keepAlive' | 'order' | 'isFrame' | 'multiTab' | 'fixedIndexInTab' | 'query'
    >;

    type Menu = Common.CommonRecord<{
      /** parent menu id */
      parentId: number;
      title: string;
      sort: number;
      visible: string;
      permission: string;
      /** menu type */
      menuType: MenuType;
      /** menu name */
      menuName: string;
      /** route path */
      path: string;
      /** component */
      component?: string;
      /** iconify icon name or local icon name */
      icon: string;
      /** icon type */
      /** buttons */
      buttons?: MenuButton[] | null;
      /** children menu */
      children?: Menu[];
      apis: [];
      menuId?: number;
    }> &
      MenuPropsOfRoute;

    /** menu list */
    type MenuList = Common.PaginatingQueryRecord<Menu>;
    /** menu search params */
    type MenuSearchParams = Partial<Pick<Api.SystemManage.Menu, 'title' | 'visible'>>;
    type MenuTree = {
      id: number;
      label: string;
      pId: number;
      children?: MenuTree[];
    };
  }
  namespace Home {
    interface commonQuery {
      pageIndex: number;
      pageSize: number;
    }
    interface bizSearchQuery {
      name: string;
      pageIndex: number;
      pageSize: number;
    }
    interface urlSearchQuery extends Omit<bizSearchQuery, 'name'> {
      url: string;
    }
    interface addAliasParams {
      alias?: string;
      contractInfo?: string;
      contractPhone?: string;
      desc?: string;
      imgUrl?: string;
      name?: string;
      // id?: number | string | null;
    }
  }
}
